# Example of TCLLk problem that does not exist in Rockit
#
# This is the labeled statements example from the TCLLk manual
# implemented with Rockit's parser combinators.
#
# The example is discussed on page 32 in the TCLLk manual as an example
# of a grammar that can be translated by hand into LL(1) but not
# by TCLLk. TCLLk can handle it but needs to create look-ahead trees
# to look ahead more than one symbol.
#
# This test is just to check if the grammar poses any troubles to Rockit.
#
require 'rockit/parse/combinator_parsers'
require 'rockit/parse/std_library'
include Rockit::Parse

int = plus(Digit)
exp = plus(Letter)

statement = int & "=" & exp
label = mult(int & ":") 

labeled_statement = label & statement

start = labeled_statement

white_space = skip(mult(" "))

start.set_separator(white_space, [int, exp])

p start.parse("1: 2 = abc")
p start.parse("1: 3: 2 = abc")
