require 'rockit/parse/combinator_parsers'
require 'rockit/parse/std_library'
include Rockit::Parse
include Rockit

valid_ascii = (0x20..0x7e).to_a.select {|c| c != '"'[0] and c != "\\"[0]}
string_character = choice(*valid_ascii.map{|c| string(c.chr)})
dquote = skip(string('"'))
string_literal = seq(dquote, mult(string_character), dquote)

ws = skip(plus(choice(string(" "), string("\t"))))
white_space = skip(mult(choice(string(" "), string("\t"))))

integer = plus("0".."9")

Int = Term.new_sub_class(:Int, :lexeme)
Identifier = Term.new_sub_class(:Identifier, :lexeme)
StringLit = Term.new_sub_class(:StringLit, :lexeme)
Print = Term.new_sub_class(:Print, :expr)
PrintLn = Term.new_sub_class(:PrintLn)
Read = Term.new_sub_class(:Read, :identifier)

newline = skip("\r\n" | "\n")
identifier = plus(choice("a".."z", "A".."Z")).build(Identifier)
expr = integer.build(Int)                                            \
     | identifier                                                    \
     | string_literal.build(StringLit)
statement = (skip('PRINT') & ws & expr       & newline).build(Print) \
          | (skip('READ')  & ws & identifier & newline).build(Read)  \
          | skip('PRINTLN').build(PrintLn)
RdgBasicParser = plus(statement)

class RdgBasicEvaluator
  def initialize
    @variables = Hash.new
  end
  
  def evaluate(res)
    case res
    when Array
      res.each {|s| evaluate(s)}
    when PrintLn
      puts ""; STDOUT.flush
    when Read
      print "? "; STDOUT.flush
      @variables[res.identifier.lexeme] = STDIN.gets.strip
    when Print
      print evaluate(res.expr)
    when Int
      res.lexeme.to_i
    when Identifier
      @variables[res.lexeme]
    when StringLit
      res.lexeme
    end
  end
end

if $0 == __FILE__
  File.open(ARGV[0], "r") do |f| 
    ast = RdgBasicParser.parse(f.read)
    RdgBasicEvaluator.new.evaluate(ast)
  end
end
