# Character sets represented as 256 bits
# $Id: char_set.rb,v 1.3 2002/01/06 21:19:02 feldt Exp $
#
# Copyright (c) 2001 Robert Feldt, feldt@ce.chalmers.se
#
module Rockit
  module Parse
    # A CharSet represents a set of characters but can also include epsilon
    # , the "empty" character.
    class CharSet
      # Represents a character set as 256 bits with ones for the characters
      # included in the set.
      def initialize(string = "", includeEpsilon = false)
	@chars, @bits = "", "\000" * 32
	@include_epsilon = includeEpsilon
	add_chars(string)
      end

      def length
	@chars.length
      end

      def include_char?(char)
	if char
	  @bits[char >> 3] & (1 << (char & 0x07)) > 0
	else
	  @include_epsilon
	end
      end

      def to_string
	@chars # Note that we loose info about epsilon!
      end

      def inspect
	"CharSet(#{@chars}" + 
	  (@include_epsilon ? " and epsilon" : "") + ")"
      end

      def invert
	self.type.new(all_chars_not_in_string(@chars), @include_epsilon)
      end

      def ==(other)
	self.type == other.type &&
	  self.bits == other.bits &&
	  self.include_epsilon? == other.include_epsilon?
      end

      def merge(set)
	set.to_string.each_byte {|char| add_char(char)}
	self.add_epsilon if set.include_epsilon?
	self
      end

      def include_epsilon?
	@include_epsilon
      end

      def add_epsilon
	@include_epsilon = true
      end

      def remove_epsilon
	@include_epsilon = false
      end

      protected

      def bits
	@bits
      end

      private

      def all_chars_not_in_string(string)
	chars = ""
	(0..0xff).each {|char| chars << char unless string.include?(char)}
	chars
      end

      def add_chars(string)
	string.each_byte {|c| add_char(c)}
      end

      def add_char(char)
	unless @chars.include?(char)
	  @bits[char >> 3] |= (1 << (char & 0x07))
	  @chars << char
	end
      end
    end
  end
end
