# TODO: Change hierarchy and names to:
#   ParseError - base class for all parse errors
#     UnexpectedInputError - base class for all errors relating to unexp. input
#       UnexpectedEosError - unexpected end of stream
#     NoMatchError - when a parser does not match
#     CustomParseError - user can customize error message
#
#  The current ParseError should be UnexpectedInputError.
#       
module Rockit
  module Parse
    #=========================================================================
    # ParseError class
    class ParseError < Exception
      def initialize(unexpected, expected, position)
	@unexp, @expected, @position = unexpected, expected, position
	super(make_message)
      end

      def unexpected_message(aStringOrChar)
	if String === aStringOrChar
	  "unexpected " + aStringOrChar
	else
	  "unexpected " + "'#{aStringOrChar.chr}'"
	end
      end

      def expected_message(exp)
	e = exp.compact.map do |e| 
	  if String === e
	    e
	  else
	    e == 0 ? nil : "'#{e.chr}'"
	  end
	end.compact
	if e.length > 1
	  "when expecting " + e[0...-1].join(", ") + " or " + e.last
	elsif e.length > 0
	  "when expecting " + e.first
	end
      end

      def make_message
	"Parse error at #{@position.inspect}:\n" +
	  [unexpected_message(@unexp), expected_message(@expected)].compact.join("\n")
      end
      
      def merge_components(error1, error2)
	if error1.position == error2.position
	  unexpmsg = error1.unexp
	  exp = (error1.expected + error2.expected).uniq
	  pos = error1.position
	else
	  if position < error2.position
	    unexpmsg = error2.unexp
	    exp = error2.expected
	    pos = error2.position
	  else
	    unexpmsg = error1.unexp
	    exp = error1.expected
	    pos = error1.position
	  end
	end
	return unexpmsg, exp, pos
      end

      def merge(other)
	unless self.type === other
	  raise "Cannot merge with non-#{self.type.inspect} object (#{other.inspect})"
	end
	self.type.new(*merge_components(self, other))
      end

      def error_class(e1, e2)
	if e1.type == e2.type
	  e1.type
	else
	  ParseError
	end
      end

      def current_char(aState)
	if aState.current
	  "'#{aState.current.chr}'"
	else
	  "end of stream"
	end
      end

      protected
      attr_reader :unexp, :expected, :position
    end # ParseError

    #=========================================================================
    # UnexpectedEosError class
    class UnexpectedEosError < ParseError
      def initialize(aParserOrString, aStateOrExpected, aPosition = nil)
	if Array === aStateOrExpected
	  raise "No position given" unless aPosition
	  super(aParserOrString, aStateOrExpected, aPosition)
	else
	  super("end of stream", aParserOrString.expected, 
		aStateOrExpected.position)
	end
      end
    end # UnexpectedEosError

    #=========================================================================
    # NoMatchError class
    class NoMatchError < ParseError
      def initialize(aParserOrString, aStateOrExpected, aPosition = nil)
	if Array === aStateOrExpected
	  raise "No position given" unless aPosition
	  super(aParserOrString, aStateOrExpected, aPosition)
	else
	  super(current_char(aStateOrExpected), aParserOrString.expected, 
		aPosition || aStateOrExpected.position)
	end
      end
    end # NoMatchError

    #=========================================================================
    # UnexpectedEosError class
    class UnexpectedError < ParseError
      def initialize(aState)
	super(current_char(aState), [], aState.position)
      end
    end # UnexpectedEosError

    #=========================================================================
    # CustomParseError class
    class CustomParseError < Exception  # ParseError
      def initialize(message, aState)
	@custom_message, @position = message, aState.position
	super(make_message)
      end

      def make_message
	"Parse error at (#{@position.inspect}):\n" +
	  @custom_message
      end
    end
  end # Parse
end # Rockit
