# Pattern matching on terms.
#
# Copyright (c) 2001, Robert Feldt, feldt@ce.chalmers.se
#
# $Id: match.rb,v 1.3 2001/12/15 00:08:09 feldt Exp $
#
require 'rockit/term/term'

module Rockit
  class Term
    class MatchData
      def initialize(*matches)
	@matches, @mismatches = matches, Array.new
      end

      def add(matchedObject)
	@matches.push matchedObject
      end

      def length; @matches.length; end
      alias size length

      def [](index); @matches[index]; end

      def mismatches
	@mismatches
      end

      def add_mismatch(expected, actual)
	@mismatches.push [expected, actual]
      end
    end

    # Try to match us against another term.
    def match(other, returnMismatches = false)
      if returnMismatches
	recursive_match_keep_mismatches(other, MatchData.new(other))
      else
	recursive_match(other, MatchData.new(other))
      end
    end

    protected

     def recursive_match(other, matchdata)
       return nil unless self.type == other.type
       val = true
       self.children.each_with_index do |child, index|
	 if child == :x || child == :X
	   matchdata.add other[index]
	 elsif Term === child
	   val = child.recursive_match(other[index], matchdata)
	 else
	   unless child == other[index]
	     puts (child.inspect + ", " + other[index].inspect)
	     return nil
	   end
	 end
	 return nil unless nil != val
       end
       matchdata
     end

    def recursive_match_keep_mismatches(other, matchdata)
      unless self.type == other.type
	matchdata.add_mismatch(self, other)
      else
	self.children.each_with_index do |child, index|
	  if child == :x || child == :X
	    matchdata.add other[index]
	  elsif Term === child
	    child.recursive_match(other[index], matchdata)
	  elsif child != other[index]
	    matchdata.add_mismatch(child, other[index]) 
	  end
	end
      end
      matchdata
    end
  end
end
